# Author: Marek K.

=begin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Dieses Programm ist Freie Software: Sie können es unter den Bedingungen
    der GNU General Public License, wie von der Free Software Foundation,
    Version 3 der Lizenz oder (nach Ihrer Wahl) jeder neueren
    veröffentlichten Version, weiter verteilen und/oder modifizieren.

    Dieses Programm wird in der Hoffnung bereitgestellt, dass es nützlich sein wird, jedoch
    OHNE JEDE GEWÄHR,; sogar ohne die implizite
    Gewähr der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
    Siehe die GNU General Public License für weitere Einzelheiten.

    Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
    Programm erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.

=end

=begin
Examlple:
hex = HexData.new [":04001000746573742C", ":0D00000048656C6C6F20576F726C642100B6", ":10004000404142434445464748494A4B4C4D4E4F38", ":00000001FF"]  # Open to read
hex.interpret  # Interpret data
p hex.ascii  # return string result of the interpret
p hex.checksum  # return checksum from the first data
p hex.checksum 1 # return checksum from the second data

hex = HexData.new  # Open to write/read
hex.push "test"  # Write "test"
hex.push_end  # Write End of File
hex.interpret  # Interpret data
p hex.ascii 0 # return string result of the interpret of the first data
=end

class HexData

  attr_accessor :push_address, :enddef

  def initialize sets=[]
    @sets = sets
    @push_address = 16
    @enddef = ":00000001FF"
  end
  
  def data
    @sets
  end
  
  def interpret
    @ascii = nil.to_a
    @sums = nil.to_a
    @sets.each { |data|
      if data[0] != ":"
        raise "Invalid data! Data: #{data}"
      end
    
      len = data[1 .. 2].to_i 16
      address = data[3 .. 6]
      type = data[7 .. 8]
      field = data[9 .. len * 2 + 8]
      sum = data[-2 .. -1].to_i 16
      str = ""
  
      csum = len + address.to_i(16) + type.to_i(16)
      #p field
      for i in (0..field.length-1).step(2)
        v = field[i .. i + 1].to_i 16
        csum += v
        str += v.chr
      end
    
      csum &= 255
      bsum = ""
      csum.to_s(2).each_char { |c| bsum += (c=="0"?"1":"0") }
      while bsum.length < 8
        bsum = "1" + bsum
      end

      if bsum.to_i(2) + 1 != sum
        raise "Invalid checksum! Data: #{data}"
      end
      @ascii << str if type == "00"
      @sums << sum
    }
  end
  
  def reset_address!
    @push_address = 16
  end
  
  def push_user str
    @sets << str
  end
  
  def delete_data! index
    @sets.delete_at index
  end
  
  def clear!
    @sets = nil.to_a
    @sums = nil.to_a
    @push_address = 16
  end
  
  def max_index?
    @sets.length
  end
  
  def data_end?
    for i in 0...@sets.length
      return i if @sets[i] == @enddef
    end
  end
  
  def checksum? index=0
    return @sums[index]
  end
  
  def ascii index=nil
    return (index ? @ascii[index] : @ascii.join(""))
  end

  def push_end
    @sets << @enddef
  end
  
  def push str
  
    if @push_address > 65535
      raise "To many data!"
    end
  
    sum = str.length + @push_address
    res = "#{str.length.to_s 16}"
    while res.length < 2
      res = "0" + res
    end
    
    res = ":" + res
    
    tmp = @push_address.to_s 16
    while tmp.length < 4
      tmp = "0" + tmp
    end
    res += tmp + "00"
    
    str.each_char { |c|
      sum += c.ord
      tmp = c.ord.to_s 16
      while tmp.length < 2
        tmp = "0" + tmp
      end
      res += tmp
    }
    
    sum &= 255
    bsum = ""
    sum.to_s(2).each_char { |c| bsum += (c=="0"?"1":"0") }
    while bsum.length < 8
      bsum = "1" + bsum
    end
  
    res += bsum.to_i(2).+(1).to_s 16
    @sets << res
    @push_address += 1
  end

end

